@@ -12,7 +12,7 @@
      * ``LC_ALL``, and
      * ``LANG``
 
-    :copyright: (c) 2013 by the Babel Team.
+    :copyright: (c) 2013-2020 by the Babel Team.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -27,7 +27,16 @@ from bisect import bisect_right
 
 from babel.core import default_locale, get_global, Locale
 from babel.util import UTC, LOCALTZ
-from babel._compat import string_types, integer_types, number_types
+from babel._compat import string_types, integer_types, number_types, PY2
+
+# "If a given short metazone form is known NOT to be understood in a given
+#  locale and the parent locale has this value such that it would normally
+#  be inherited, the inheritance of this value can be explicitly disabled by
+#  use of the 'no inheritance marker' as the value, which is 3 simultaneous [sic]
+#  empty set characters ( U+2205 )."
+#  - https://www.unicode.org/reports/tr35/tr35-dates.html#Metazone_Names
+
+NO_INHERITANCE_MARKER = u'\u2205\u2205\u2205'
 
 
 LC_TIME = default_locale('LC_TIME')
@@ -512,7 +521,7 @@ def get_timezone_location(dt_or_tzinfo=None, locale=LC_TIME, return_city=False):
         territory = 'ZZ'  # invalid/unknown
     territory_name = locale.territories[territory]
     if not return_city and territory and len(get_global('territory_zones').get(territory, [])) == 1:
-        return region_format % (territory_name)
+        return region_format % territory_name
 
     # Otherwise, include the city in the output
     fallback_format = locale.zone_formats['fallback']
@@ -583,7 +592,7 @@ def get_timezone_name(dt_or_tzinfo=None, width='long', uncommon=False,
     format.
 
     For more information see `LDML Appendix J: Time Zone Display Names
-    <http://www.unicode.org/reports/tr35/#Time_Zone_Fallback>`_
+    <https://www.unicode.org/reports/tr35/#Time_Zone_Fallback>`_
 
     .. versionadded:: 0.9
 
@@ -642,8 +651,13 @@ def get_timezone_name(dt_or_tzinfo=None, width='long', uncommon=False,
     if metazone:
         metazone_info = locale.meta_zones.get(metazone, {})
         if width in metazone_info:
-            if zone_variant in metazone_info[width]:
-                return metazone_info[width][zone_variant]
+            name = metazone_info[width].get(zone_variant)
+            if width == 'short' and name == NO_INHERITANCE_MARKER:
+                # If the short form is marked no-inheritance,
+                # try to fall back to the long name instead.
+                name = metazone_info.get('long', {}).get(zone_variant)
+            if name:
+                return name
 
     # If we have a concrete datetime, we assume that the result can't be
     # independent of daylight savings time, so we return the GMT offset
@@ -968,7 +982,7 @@ def format_interval(start, end, skeleton=None, tzinfo=None, fuzzy=True, locale=L
     u'15.\u201317.1.2016'
 
     >>> format_interval(time(12, 12), time(16, 16), "Hm", locale="en_GB")
-    '12:12 \u2013 16:16'
+    '12:12\u201316:16'
 
     >>> format_interval(time(5, 12), time(16, 16), "hm", locale="en_US")
     '5:12 AM \u2013 4:16 PM'
@@ -1004,7 +1018,7 @@ def format_interval(start, end, skeleton=None, tzinfo=None, fuzzy=True, locale=L
     locale = Locale.parse(locale)
 
     # NB: The quote comments below are from the algorithm description in
-    #     http://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
+    #     https://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
 
     # > Look for the intervalFormatItem element that matches the "skeleton",
     # > starting in the current locale and then following the locale fallback
@@ -1080,12 +1094,16 @@ def get_period_id(time, tzinfo=None, type=None, locale=LC_TIME):
     locale = Locale.parse(locale)
 
     # The LDML rules state that the rules may not overlap, so iterating in arbitrary
-    # order should be alright.
-    for rule_id, rules in locale.day_period_rules.get(type, {}).items():
+    # order should be alright, though `at` periods should be preferred.
+    rulesets = locale.day_period_rules.get(type, {}).items()
+
+    for rule_id, rules in rulesets:
         for rule in rules:
             if "at" in rule and rule["at"] == seconds_past_midnight:
                 return rule_id
 
+    for rule_id, rules in rulesets:
+        for rule in rules:
             start_ok = end_ok = False
 
             if "from" in rule and seconds_past_midnight >= rule["from"]:
@@ -1096,8 +1114,8 @@ def get_period_id(time, tzinfo=None, type=None, locale=LC_TIME):
                 end_ok = True
             if "before" in rule and seconds_past_midnight < rule["before"]:
                 end_ok = True
-            if "after" in rule and seconds_past_midnight > rule["after"]:
-                start_ok = True
+            if "after" in rule:
+                raise NotImplementedError("'after' is deprecated as of CLDR 29.")
 
             if start_ok and end_ok:
                 return rule_id
@@ -1137,7 +1155,7 @@ def parse_date(string, locale=LC_TIME):
     # FIXME: this currently only supports numbers, but should also support month
     #        names, both in the requested locale, and english
 
-    numbers = re.findall('(\d+)', string)
+    numbers = re.findall(r'(\d+)', string)
     year = numbers[indexes['Y']]
     if len(year) == 2:
         year = 2000 + int(year)
@@ -1180,7 +1198,7 @@ def parse_time(string, locale=LC_TIME):
     #        and seconds should be optional, maybe minutes too
     #        oh, and time-zones, of course
 
-    numbers = re.findall('(\d+)', string)
+    numbers = re.findall(r'(\d+)', string)
     hour = int(numbers[indexes['H']])
     minute = int(numbers[indexes['M']])
     second = int(numbers[indexes['S']])
@@ -1199,6 +1217,12 @@ class DateTimePattern(object):
     def __unicode__(self):
         return self.pattern
 
+    def __str__(self):
+        pat = self.pattern
+        if PY2:
+            pat = pat.encode('utf-8')
+        return pat
+
     def __mod__(self, other):
         if type(other) is not DateTimeFormat:
             return NotImplemented
@@ -1239,6 +1263,7 @@ class DateTimeFormat(object):
         elif char in ('E', 'e', 'c'):
             return self.format_weekday(char, num)
         elif char == 'a':
+            # TODO: Add support for the rest of the period formats (a*, b*, B*)
             return self.format_period(char)
         elif char == 'h':
             if self.value.hour % 12 == 0:
@@ -1278,7 +1303,7 @@ class DateTimeFormat(object):
         elif char == 'H':
             return self.value.hour
         elif char == 'h':
-            return (self.value.hour % 12 or 12)
+            return self.value.hour % 12 or 12
         elif char == 'm':
             return self.value.minute
         elif char == 'a':
@@ -1294,9 +1319,7 @@ class DateTimeFormat(object):
     def format_year(self, char, num):
         value = self.value.year
         if char.isupper():
-            week = self.get_week_number(self.get_day_of_year())
-            if week == 0:
-                value -= 1
+            value = self.value.isocalendar()[0]
         year = self.format(value, num)
         if num == 2:
             year = year[-2:]
@@ -1305,14 +1328,14 @@ class DateTimeFormat(object):
     def format_quarter(self, char, num):
         quarter = (self.value.month - 1) // 3 + 1
         if num <= 2:
-            return ('%%0%dd' % num) % quarter
+            return '%0*d' % (num, quarter)
         width = {3: 'abbreviated', 4: 'wide', 5: 'narrow'}[num]
         context = {'Q': 'format', 'q': 'stand-alone'}[char]
         return get_quarter_names(width, context, self.locale)[quarter]
 
     def format_month(self, char, num):
         if num <= 2:
-            return ('%%0%dd' % num) % self.value.month
+            return '%0*d' % (num, self.value.month)
         width = {3: 'abbreviated', 4: 'wide', 5: 'narrow'}[num]
         context = {'M': 'format', 'L': 'stand-alone'}[char]
         return get_month_names(width, context, self.locale)[self.value.month]
@@ -1381,7 +1404,11 @@ class DateTimeFormat(object):
 
     def format_period(self, char):
         period = {0: 'am', 1: 'pm'}[int(self.value.hour >= 12)]
-        return get_period_names(locale=self.locale)[period]
+        for width in ('wide', 'narrow', 'abbreviated'):
+            period_names = get_period_names(context='format', width=width, locale=self.locale)
+            if period in period_names:
+                return period_names[period]
+        raise ValueError('Could not format period %s in %s' % (period, self.locale))
 
     def format_frac_seconds(self, num):
         """ Return fractional seconds.
@@ -1441,7 +1468,7 @@ class DateTimeFormat(object):
                 return get_timezone_gmt(self.value, width='iso8601', locale=self.locale)
 
     def format(self, value, length):
-        return ('%%0%dd' % length) % value
+        return '%0*d' % (length, value)
 
     def get_day_of_year(self, date=None):
         if date is None:
@@ -1476,8 +1503,20 @@ class DateTimeFormat(object):
         if first_day < 0:
             first_day += 7
         week_number = (day_of_period + first_day - 1) // 7
+
         if 7 - first_day >= self.locale.min_week_days:
             week_number += 1
+
+        if self.locale.first_week_day == 0:
+            # Correct the weeknumber in case of iso-calendar usage (first_week_day=0).
+            # If the weeknumber exceeds the maximum number of weeks for the given year
+            # we must count from zero.For example the above calculation gives week 53
+            # for 2018-12-31. By iso-calender definition 2018 has a max of 52
+            # weeks, thus the weeknumber must be 53-52=1.
+            max_weeks = date(year=self.value.year, day=28, month=12).isocalendar()[1]
+            if week_number > max_weeks:
+                week_number -= max_weeks
+
         return week_number
 
 
@@ -1498,7 +1537,7 @@ PATTERN_CHARS = {
 }
 
 #: The pattern characters declared in the Date Field Symbol Table
-#: (http://www.unicode.org/reports/tr35/tr35-dates.html#Date_Field_Symbol_Table)
+#: (https://www.unicode.org/reports/tr35/tr35-dates.html#Date_Field_Symbol_Table)
 #: in order of decreasing magnitude.
 PATTERN_CHAR_ORDER = "GyYuUQqMLlwWdDFgEecabBChHKkjJmsSAzZOvVXx"
 
@@ -1646,7 +1685,7 @@ def split_interval_pattern(pattern):
     Split an interval-describing datetime pattern into multiple pieces.
 
     > The pattern is then designed to be broken up into two pieces by determining the first repeating field.
-    - http://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
+    - https://www.unicode.org/reports/tr35/tr35-dates.html#intervalFormats
 
     >>> split_interval_pattern(u'E d.M. \u2013 E d.M.')
     [u'E d.M. \u2013 ', 'E d.M.']
