@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2020 the Babel team
 # All rights reserved.
 #
-# This software is licensed as described in the file COPYING, which
+# This software is licensed as described in the file LICENSE, which
 # you should have received as part of this distribution. The terms
 # are also available at http://babel.edgewall.org/wiki/License.
 #
@@ -15,8 +15,9 @@ import copy
 import datetime
 import unittest
 
+from babel._compat import StringIO
 from babel.dates import format_datetime, UTC
-from babel.messages import catalog
+from babel.messages import catalog, pofile
 from babel.util import FixedOffsetTimezone
 
 
@@ -44,9 +45,9 @@ class MessageTestCase(unittest.TestCase):
         self.assertEqual(mess.user_comments, ['Comment About `foo`'])
         mess = catalog.Message('foo',
                                auto_comments=['Comment 1 About `foo`',
-                                         'Comment 2 About `foo`'])
+                                              'Comment 2 About `foo`'])
         self.assertEqual(mess.auto_comments, ['Comment 1 About `foo`',
-                                         'Comment 2 About `foo`'])
+                                              'Comment 2 About `foo`'])
 
     def test_clone_message_object(self):
         msg = catalog.Message('foo', locations=[('foo.py', 42)])
@@ -58,10 +59,11 @@ class MessageTestCase(unittest.TestCase):
 
 
 class CatalogTestCase(unittest.TestCase):
+
     def test_add_returns_message_instance(self):
         cat = catalog.Catalog()
         message = cat.add('foo')
-        self.assertEquals('foo', message.id)
+        self.assertEqual('foo', message.id)
 
     def test_two_messages_with_same_singular(self):
         cat = catalog.Catalog()
@@ -100,7 +102,7 @@ class CatalogTestCase(unittest.TestCase):
 
     def test_update_message_changed_to_simple(self):
         cat = catalog.Catalog()
-        cat.add((u'foo' u'foos'), (u'Voh', u'Vöhs'))
+        cat.add(u'foo' u'foos', (u'Voh', u'Vöhs'))
         tmpl = catalog.Catalog()
         tmpl.add(u'foo')
         cat.update(tmpl)
@@ -254,7 +256,7 @@ class CatalogTestCase(unittest.TestCase):
         self.assertEqual(template.creation_date,
                          localized_catalog.creation_date)
         template.creation_date = datetime.datetime.now() - \
-                                                datetime.timedelta(minutes=5)
+            datetime.timedelta(minutes=5)
         localized_catalog.update(template)
         self.assertEqual(template.creation_date,
                          localized_catalog.creation_date)
@@ -270,7 +272,7 @@ class CatalogTestCase(unittest.TestCase):
         self.assertEqual(template.creation_date,
                          localized_catalog.creation_date)
         template.creation_date = datetime.datetime.now() - \
-                                                datetime.timedelta(minutes=5)
+            datetime.timedelta(minutes=5)
         localized_catalog.update(template)
         self.assertEqual(localized_catalog.revision_date, fake_rev_date)
 
@@ -278,8 +280,8 @@ class CatalogTestCase(unittest.TestCase):
         localized = catalog.Catalog()
         localized.locale = 'de_DE'
         localized[''] = catalog.Message('',
-                       "POT-Creation-Date: 2009-03-09 15:47-0700\n" +
-                       "PO-Revision-Date: 2009-03-09 15:47-0700\n")
+                                        "POT-Creation-Date: 2009-03-09 15:47-0700\n" +
+                                        "PO-Revision-Date: 2009-03-09 15:47-0700\n")
         for key, value in localized.mime_headers:
             if key in ('POT-Creation-Date', 'PO-Revision-Date'):
                 self.assertEqual(value, '2009-03-09 15:47-0700')
@@ -287,10 +289,10 @@ class CatalogTestCase(unittest.TestCase):
     def test_mime_headers_contain_same_information_as_attributes(self):
         cat = catalog.Catalog()
         cat[''] = catalog.Message('',
-                      "Last-Translator: Foo Bar <foo.bar@example.com>\n" +
-                      "Language-Team: de <de@example.com>\n" +
-                      "POT-Creation-Date: 2009-03-01 11:20+0200\n" +
-                      "PO-Revision-Date: 2009-03-09 15:47-0700\n")
+                                  "Last-Translator: Foo Bar <foo.bar@example.com>\n" +
+                                  "Language-Team: de <de@example.com>\n" +
+                                  "POT-Creation-Date: 2009-03-01 11:20+0200\n" +
+                                  "PO-Revision-Date: 2009-03-09 15:47-0700\n")
         self.assertEqual(None, cat.locale)
         mime_headers = dict(cat.mime_headers)
 
@@ -313,10 +315,12 @@ def test_message_fuzzy():
     assert msg.fuzzy
     assert msg.id == 'foo'
 
+
 def test_message_pluralizable():
     assert not catalog.Message('foo').pluralizable
     assert catalog.Message(('foo', 'bar')).pluralizable
 
+
 def test_message_python_format():
     assert catalog.Message('foo %(name)s bar').python_format
     assert catalog.Message(('foo %(name)s', 'foo %(name)s')).python_format
@@ -329,7 +333,7 @@ def test_catalog():
         '# Translations template for Foobar.\n'
         '# Copyright (C) %(year)d Foo Company\n'
         '# This file is distributed under the same '
-            'license as the Foobar project.\n'
+        'license as the Foobar project.\n'
         '# FIRST AUTHOR <EMAIL@ADDRESS>, %(year)d.\n'
         '#') % {'year': datetime.date.today().year}
 
@@ -392,13 +396,13 @@ def test_catalog_mime_headers_set_locale():
 
 def test_catalog_num_plurals():
     assert catalog.Catalog(locale='en').num_plurals == 2
-    assert catalog.Catalog(locale='ga').num_plurals == 3
+    assert catalog.Catalog(locale='ga').num_plurals == 5
 
 
 def test_catalog_plural_expr():
     assert catalog.Catalog(locale='en').plural_expr == '(n != 1)'
     assert (catalog.Catalog(locale='ga').plural_expr
-            == '(n==1 ? 0 : n==2 ? 1 : 2)')
+            == '(n==1 ? 0 : n==2 ? 1 : n>=3 && n<=6 ? 2 : n>=7 && n<=10 ? 3 : 4)')
 
 
 def test_catalog_plural_forms():
@@ -436,11 +440,10 @@ def test_catalog_update():
     cat.add('blue', u'blau', locations=[('main.py', 98)])
     cat.add('head', u'Kopf', locations=[('util.py', 33)])
     cat.add(('salad', 'salads'), (u'Salat', u'Salate'),
-                locations=[('util.py', 38)])
+            locations=[('util.py', 38)])
 
     cat.update(template)
     assert len(cat) == 3
-    assert cat.header_comment == template.header_comment  # Header comment also gets updated
 
     msg1 = cat['green']
     msg1.string
@@ -454,9 +457,12 @@ def test_catalog_update():
     assert msg3.string == (u'Salat', u'Salate')
     assert msg3.locations == [('util.py', 42)]
 
-    assert not 'head' in cat
+    assert 'head' not in cat
     assert list(cat.obsolete.values())[0].id == 'head'
 
+    cat.update(template, update_header_comment=True)
+    assert cat.header_comment == template.header_comment  # Header comment also gets updated
+
 
 def test_datetime_parsing():
     val1 = catalog._parse_datetime_header('2006-06-28 23:24+0200')
@@ -470,3 +476,31 @@ def test_datetime_parsing():
     assert val2.month == 6
     assert val2.day == 28
     assert val2.tzinfo is None
+
+
+def test_update_catalog_comments():
+    # Based on https://web.archive.org/web/20100710131029/http://babel.edgewall.org/attachment/ticket/163/cat-update-comments.py
+
+    catalog = pofile.read_po(StringIO('''
+    # A user comment
+    #. An auto comment
+    #: main.py:1
+    #, fuzzy, python-format
+    msgid "foo %(name)s"
+    msgstr "foo %(name)s"
+    '''))
+
+    assert all(message.user_comments and message.auto_comments for message in catalog if message.id)
+
+    # NOTE: in the POT file, there are no comments
+    template = pofile.read_po(StringIO('''
+    #: main.py:1
+    #, fuzzy, python-format
+    msgid "bar %(name)s"
+    msgstr ""
+    '''))
+
+    catalog.update(template)
+
+    # Auto comments will be obliterated here
+    assert all(message.user_comments for message in catalog if message.id)
