@@ -1,9 +1,9 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2011 Edgewall Software
+# Copyright (C) 2007-2011 Edgewall Software, 2013-2020 the Babel team
 # All rights reserved.
 #
-# This software is licensed as described in the file COPYING, which
+# This software is licensed as described in the file LICENSE, which
 # you should have received as part of this distribution. The terms
 # are also available at http://babel.edgewall.org/wiki/License.
 #
@@ -24,6 +24,8 @@ from babel.messages import Catalog
 from babel.messages.mofile import write_mo
 from babel._compat import BytesIO, PY2
 
+get_arg_spec = (inspect.getargspec if PY2 else inspect.getfullargspec)
+
 
 @pytest.mark.usefixtures("os_environ")
 class TranslationsTestCase(unittest.TestCase):
@@ -186,6 +188,7 @@ class TranslationsTestCase(unittest.TestCase):
 
 
 class NullTranslationsTestCase(unittest.TestCase):
+
     def setUp(self):
         fp = BytesIO()
         write_mo(fp, Catalog(locale='de'))
@@ -205,9 +208,10 @@ class NullTranslationsTestCase(unittest.TestCase):
         for name in self.method_names():
             translations_method = getattr(self.translations, name)
             null_method = getattr(self.null_translations, name)
-            signature = inspect.getargspec
-            self.assertEqual(signature(translations_method),
-                             signature(null_method))
+            self.assertEqual(
+                get_arg_spec(translations_method),
+                get_arg_spec(null_method),
+            )
 
     def test_same_return_values(self):
         data = {
@@ -218,15 +222,17 @@ class NullTranslationsTestCase(unittest.TestCase):
         for name in self.method_names():
             method = getattr(self.translations, name)
             null_method = getattr(self.null_translations, name)
-            signature = inspect.getargspec(method)
-            parameter_names = [name for name in signature[0] if name != 'self']
+            signature = get_arg_spec(method)
+            parameter_names = [name for name in signature.args if name != 'self']
             values = [data[name] for name in parameter_names]
             self.assertEqual(method(*values), null_method(*values))
 
 
 class LazyProxyTestCase(unittest.TestCase):
+
     def test_proxy_caches_result_of_function_call(self):
         self.counter = 0
+
         def add_one():
             self.counter += 1
             return self.counter
@@ -236,6 +242,7 @@ class LazyProxyTestCase(unittest.TestCase):
 
     def test_can_disable_proxy_cache(self):
         self.counter = 0
+
         def add_one():
             self.counter += 1
             return self.counter
@@ -246,7 +253,8 @@ class LazyProxyTestCase(unittest.TestCase):
     def test_can_copy_proxy(self):
         from copy import copy
 
-        numbers = [1,2]
+        numbers = [1, 2]
+
         def first(xs):
             return xs[0]
 
@@ -259,7 +267,8 @@ class LazyProxyTestCase(unittest.TestCase):
 
     def test_can_deepcopy_proxy(self):
         from copy import deepcopy
-        numbers = [1,2]
+        numbers = [1, 2]
+
         def first(xs):
             return xs[0]
 
@@ -352,4 +361,4 @@ def test_catalog_merge_files():
     t2._catalog["bar"] = "quux"
     t1.merge(t2)
     assert t1.files == ["pro.mo"]
-    assert set(t1._catalog.keys()) == set(('', 'foo', 'bar'))
+    assert set(t1._catalog.keys()) == {'', 'foo', 'bar'}
